function [ z, mu, K ] = dpmeans( x, lambda, maxK, maxT, eps, quiet )

    n = size(x, 1);
    D = size(x, 2);
    
    z = ones(n, 1);
    mu = zeros(maxK, D);
    mu(1, :) = mean(x);
    
    Q = zeros(n, maxK);
    mu1 = mu(1, :); % 1 x D
    Q(:, 1) = sum((x - mu1(ones(n, 1), :)) .^ 2, 2);
    for k = 2:maxK
        Q(:, k) = 1e100 * ones(n, 1);
    end
    K = 1;
    
    lastloss = 1e100;
    loss = lambda + sum(Q(:, 1));
    if (quiet ~= 1)
        fprintf('Init loss = %f\n', loss);
    end
    
    T = 0;
    while (T < maxT && abs(lastloss - loss) / lastloss > eps)
        
        T = T + 1;
        lastloss = loss;
        
        % update z
        [min_value, z] = min(Q, [], 2);
        while (K < maxK && max(min_value) > lambda)
            [max_value, idx] = max(min_value);
            K = K + 1;
            muk = x(idx, :); % 1 x D
            mu(K, :) = muk;
            Q(:, K) = sum((x - muk(ones(n, 1), :)) .^ 2, 2);
            [min_value, z] = min(Q, [], 2);
        end
        
        % update mu
        for k = 1:K
            ind = (z == k * ones(n, 1)); % n x 1
            muk = sum(ind(:, ones(1, D)) .* x) / sum(ind);
            mu(k, :) = muk;
        end
        
        % update cost
        for k = 1:K
            muk = mu(k, :);
            Q(:, k) = sum((x - muk(ones(n, 1), :)) .^ 2, 2);
        end
        for k = K+1:maxK
            Q(:, k) = 1e100 * ones(n, 1);
        end
        
        loss = lambda * K + sum(min(Q, [], 2));
        if (quiet ~= 1)
            fprintf('Iter %d: K = %d, loss = %f\n', T, K, loss);
        end
        
    end

end

