% validate for UCI datasets

n = size(x, 1);
m = size(x, 2);
K0 = max(z);
T = 10;
maxK = 20;
maxT = 100;

lambdas = [0.001 0.0025 0.005 0.01 0.025 0.05 0.1 0.25 0.5 1 2.5 5 10 25 50 100 250 500 1000];
ss = [0.001 0.0025 0.005 0.01 0.025 0.05 0.1 0.25 0.5 1 2.5 5 10 25 50 100 250 500 1000];

ds = zeros(1, m);
for i = 1:5
    ds(i) = i - 1;
end

zT = zeros(4, T, n);
dT = zeros(4, T);
rT = zeros(4, T);
nmiT = zeros(4, T);
KT = zeros(4, T);

% the validate dataset
rate = 1.0;
n0 = int32(n * rate);
x0 = x(1:n0, :);
z0 = z(1:n0);

% Algo1: DPspace
best_lambda = 0;
best_s = 0;
best_nmi = 0;
for i = 1:size(lambdas, 2)
    fprintf('i = %d\n', i);
    for j = 1:size(ss, 2)
        lambda = lambdas(i);
        s = ss(j);
        [z1, K1, w1, mu1, d1] = dpspace(x0, lambda, s, 1, maxK, maxT, 1e-4, 1);
        nmi1 = compute_nmi(z1, z0);
        
        if (nmi1 > best_nmi)
            best_lambda = lambda;
            best_s = s;
            best_nmi = nmi1;
        end
    end
end

% now the real run
for t = 1:T
    tt = cputime;
    [z1, K1, w1, mu1, d1] = dpspace(x, best_lambda, best_s / rate, 1, maxK, maxT, 1e-4, 0);
    rT(1, t) = cputime - tt;
    zT(1, t, :) = z1;
    nmiT(1, t) = compute_nmi(z1, z);
    dT(1, t) = mean(d1);
    KT(1, t) = max(z1);
end


% Algo2: MPPCA
best_d = 0;
best_nmi = 0;
for i = 1:size(ds, 2);
    d = ds(i);
    [z1, pi1, w1, mu1, sigmasq1] = em_mppca(x0, K0, d * ones(K0, 1), maxT, 1e-4);
    nmi1 = compute_nmi(z1, z0);
    if (nmi1 > best_nmi)
        best_d = d;
        best_nmi = nmi1;
    end
end

% now the real run
for t = 1:T
    tt = cputime;
    [z1, pi1, w1, mu1, sigmasq1] = em_mppca(x, K0, best_d * ones(K0, 1), maxT, 1e-4);
    rT(2, t) = cputime - tt;
    zT(2, t, :) = z1;
    nmiT(2, t) = compute_nmi(z1, z);
    dT(2, t) = best_d;
    KT(2, t) = K0;
end

% Algo3: K-means
for t = 1:T
    tt = cputime;
    [z1, mu1] = kmeans(x, K0, 50, 1e-4);
    rT(3, t) = cputime - tt;
    zT(3, t, :) = z1;
    nmiT(3, t) = compute_nmi(z1, z);
    
    while (nmiT(3, t) == 0)
        tt = cputime;
        [z1, mu1] = kmeans(x, K0, 50, 1e-4);
        rT(3, t) = cputime - tt;
        zT(3, t, :) = z1;
        nmiT(3, t) = compute_nmi(z1, z);
    end
    
    KT(3, t) = K0;
end

% Algo4: DPmeans
best_lambda = 0;
best_nmi = 0;
for i = 1:size(lambdas, 2);
    lambda = lambdas(i);
    [z1, mu1, K1] = dpmeans(x0, lambda, maxK, maxT, 1e-4, 1);
    nmi1 = compute_nmi(z1, z0);
    if (nmi1 > best_nmi)
        best_lambda = lambda;
        best_nmi = nmi1;
    end
end

% the real run
for t = 1:T
    tt = cputime;
    [z1, mu1, K1] = dpmeans(x, best_lambda, maxK, maxT, 1e-4, 1);
    rT(4, t) = cputime - tt;
    zT(4, t, :) = z1;
    nmiT(4, t) = compute_nmi(z1, z);
    KT(4, t) = max(z1);
end

